#!/usr/bin/env node
'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _child_process = require('child_process');

var _rawBody = require('raw-body');

var _rawBody2 = _interopRequireDefault(_rawBody);

var _jsYaml = require('js-yaml');

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var _dockerParseImage = require('docker-parse-image');

var _dockerParseImage2 = _interopRequireDefault(_dockerParseImage);

var _requestPromise = require('request-promise');

var _requestPromise2 = _interopRequireDefault(_requestPromise);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _ttys = require('ttys');

var _ttys2 = _interopRequireDefault(_ttys);

var _deepEqual = require('deep-equal');

var _deepEqual2 = _interopRequireDefault(_deepEqual);

var _manifest = require('./manifest');

var _buildRequests = require('./build-requests');

var _buildRequests2 = _interopRequireDefault(_buildRequests);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new _bluebird2.default(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return _bluebird2.default.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const { log } = console;

const getConfig = require('./get-config')({ exec: _child_process.exec });
const getOptions = require('./get-options')({ process, minimist: _minimist2.default });
const parseInputYaml = require('./parse-input-yaml')({ process, fs: _fs2.default, rawBody: _rawBody2.default, jsYaml: _jsYaml2.default });
const listTags = require('./list-tags')({ request: _requestPromise2.default });
const submitBuildRequest = require('./submit-build-request')({ request: _requestPromise2.default });
const manifestContainerBuilds = require('./manifest-container-builds')({ getContainers: _manifest.getContainers, getBuilds: _manifest.getBuilds });
const compileRequiredImages = require('./compile-required-images')({ isSupported: _manifest.isSupported, manifestContainerBuilds, compareBuilds: deepEqual });
const filterMissingImages = require('./filter-missing-images')({ listTags });
const inquirerModule = _inquirer2.default.createPromptModule({ input: _ttys2.default.stdin, output: _ttys2.default.stdout });
const selectImages = require('./select-images')({ inquirer: inquirerModule });
const confirmSubmit = require('./confirm-submit')({ log, inquirer: inquirerModule });
const logBuild = require('./log-build')({ process, exec: _child_process.exec });

(() => {
    var _ref = _asyncToGenerator(function* () {
        try {
            const { projectId, token } = yield getConfig();
            const { file, repoName } = getOptions();
            const manifests = yield parseInputYaml(file);
            const requiredImages = compileRequiredImages(projectId, manifests);
            const missingImages = yield filterMissingImages(requiredImages, projectId, token);
            if (!missingImages.length) {
                process.exit(0);
            }
            const selected = yield selectImages(missingImages);
            if (!selected.length) {
                process.exit(0);
            }
            const requests = (0, _buildRequests2.default)({ projectId, repoName, containerBuilds: selected });
            const { confirmed } = yield confirmSubmit(requests);
            if (!confirmed) {
                process.exit(1);
            }
            for (let i = 0; i < requests.length; i++) {
                const request = requests[i];
                const id = yield submitBuildRequest({ buildRequest: request, projectId, token });
                yield logBuild(id);
            }
        } catch (e) {
            console.log(e);
            process.exit(1);
        }
    });

    function main() {
        return _ref.apply(this, arguments);
    }

    return main;
})()();

function deepEqual(a, b) {
    return (0, _deepEqual2.default)(a, b, { strict: true });
}